#include "gitSettingsDlg.h"
#include "windowattrmanager.h"
#include "cl_config.h"
#include "gitentry.h"
#include "event_notifier.h"

GitSettingsDlg::GitSettingsDlg(wxWindow* parent, const wxString& localRepoPath)
    : GitSettingsDlgBase(parent)
    , m_localRepoPath(localRepoPath)
{
    clConfig conf("git.conf");
    GitEntry data;
    conf.ReadItem(&data);

    m_pathGIT->SetPath(data.GetGITExecutablePath());
    m_pathGITK->SetPath(data.GetGITKExecutablePath());
    m_checkBoxLog->SetValue( data.GetFlags() & GitEntry::Git_Verbose_Log );
    m_checkBoxTerminal->SetValue( data.GetFlags() & GitEntry::Git_Show_Terminal );
    m_checkBoxTrackTree->SetValue( data.GetFlags() & GitEntry::Git_Colour_Tree_View );

    GitEntry::GitProperties props = GitEntry::ReadGitProperties( m_localRepoPath );

    m_textCtrlGlobalEmail->ChangeValue( props.global_email );
    m_textCtrlGlobalName->ChangeValue( props.global_username );
    m_textCtrlLocalEmail->ChangeValue( props.local_email );
    m_textCtrlLocalName->ChangeValue( props.local_username );

    WindowAttrManager::Load(this, wxT("GitSettingsDlg"), NULL);
}

GitSettingsDlg::~GitSettingsDlg()
{
    WindowAttrManager::Save(this, wxT("GitSettingsDlg"), NULL);
}

void GitSettingsDlg::OnOK(wxCommandEvent& event)
{
    clConfig conf("git.conf");
    GitEntry data;
    conf.ReadItem(&data);
    data.SetGITExecutablePath( m_pathGIT->GetPath() );
    data.SetGITKExecutablePath( m_pathGITK->GetPath() );

    size_t flags = 0;
    if ( m_checkBoxLog->IsChecked() )
        flags |= GitEntry::Git_Verbose_Log;

    if ( m_checkBoxTerminal->IsChecked() )
        flags |= GitEntry::Git_Show_Terminal;

    if ( m_checkBoxTrackTree->IsChecked())
        flags |= GitEntry::Git_Colour_Tree_View;

    data.SetFlags( flags );
    conf.WriteItem(&data);

    GitEntry::GitProperties props;
    props.global_email = m_textCtrlGlobalEmail->GetValue();
    props.global_username = m_textCtrlGlobalName->GetValue();
    props.local_email = m_textCtrlLocalEmail->GetValue();
    props.local_username = m_textCtrlLocalName->GetValue();
    GitEntry::WriteGitProperties(m_localRepoPath, props);

    // Notify about configuration changed
    wxCommandEvent evt(wxEVT_GIT_CONFIG_CHANGED);
    EventNotifier::Get()->AddPendingEvent( evt );

    EndModal(wxID_OK);
}

void GitSettingsDlg::OnLocalRepoUI(wxUpdateUIEvent& event)
{
    event.Enable( !m_localRepoPath.IsEmpty() );
}
